
--------------------------------------------------------------
-- CREACIO LES TAULES  i TRIGGERS --
-- M.MERC LLORET I ROSSELL
---------------------------------------------------------------- 

-- APLICACIONS
CREATE TABLE APLICACIONS (
  idAplicacio    VARCHAR2(40) NOT NULL,
  versio         VARCHAR2(20) NOT NULL,
  dataPujada     DATE NOT NULL,
  url            VARCHAR2(200),
  resolucio      VARCHAR2(20) NOT NULL,
  activa         CHAR(2) NOT NULL, 
  CONSTRAINT PK_APLICACIONs PRIMARY KEY (idAplicacio)
);

-- PAIS
CREATE TABLE PAIS (
  idPais         CHAR(2) NOT NULL,
  nomPais        VARCHAR2(50) NOT NULL,
  CONSTRAINT PK_PAIS PRIMARY KEY (idPais)
);

-- PREU
CREATE TABLE PREU(
  aplicacio     VARCHAR2(40) NOT NULL,
  pais          CHAR(2) NOT NULL,
  preu          NUMBER,
  CONSTRAINT PK_PREU PRIMARY KEY (aplicacio, pais),
  CONSTRAINT FK_PREU_APLICACIO FOREIGN KEY (aplicacio) REFERENCES APLICACIONS(idAplicacio),
  CONSTRAINT FK_PREu_PAIS FOREIGN KEY (pais) REFERENCES PAIS(idPais)
);

-- DESCRIPCIO
CREATE TABLE DESCRIPCIO (
  idioma          VARCHAR2(20) NOT NULL,
  aplicacio       VARCHAR2(40) NOT NULL,
  descripcio      VARCHAR2(300) NOT NULL,
  CONSTRAINT PK_DESCRIPCIO PRIMARY KEY (idioma, aplicacio),
  CONSTRAINT FK_DESCRIPCIO_APLICACIO FOREIGN KEY (aplicacio) REFERENCES APLICACIONS(idAplicacio)
);

-- SISTEMA OPERATIU
CREATE TABLE SOPERATIU (
  idSO           VARCHAR2(20) NOT NULL,
  nom VARCHAR2(50) NOT NULL,
  CONSTRAINT PK_SOPERATIU PRIMARY KEY (idSO)
);

-- DISPONIBIBLITAT
CREATE TABLE DISPONIBILITAT (
  aplicacio     VARCHAR2(40) NOT NULL,
  soperatiu     VARCHAR2(20) NOT NULL,
  mida          NUMBER NOT NULL,
  enllacbinari  VARCHAR2(100) NOT NULL, 
  CONSTRAINT PK_DISPONIBILITAT PRIMARY KEY (aplicacio, soperatiu),
  CONSTRAINT FK_DISPONIBILITAT_APLICACIO FOREIGN KEY (aplicacio) REFERENCES APLICACIONS(idAplicacio),
  CONSTRAINT FK_DISPONIBILITAT_SOPERATIU FOREIGN KEY (soperatiu) REFERENCES SOPERATIU(idSO)
);

-- USUARI
CREATE TABLE USUARI (
  nmobil      NUMBER NOT NULL,
  operador    VARCHAR2(20) NOT NULL,
  correu      VARCHAR2(40) NOT NULL,
  pais        CHAR(2) NOT NULL,
  CONSTRAINT PK_USUARI PRIMARY KEY (nmobil),
  CONSTRAINT FK_USUARI_PAIS FOREIGN KEY (pais) REFERENCES PAIS(idPais)
);


-- DESENVOLUPADOR
CREATE TABLE DESENVOLUPADOR(
  idDesenvolupador      NUMBER NOT NULL,
  nom                   VARCHAR2(50) NOT NULL,
  nomRepresentant       VARCHAR2(50) NOT NULL,
  pais                  CHAR(2) NOT NULL,
  adreca                VARCHAR2(70) NOT NULL,
  telefon               NUMBER NOT NULL,
  
  CONSTRAINT PK_DESENVOLUPADOR PRIMARY KEY (idDesenvolupador) USING INDEX TABLESPACE TS_INDEXS,
  CONSTRAINT FK_DESENVOLUPADOR_PAIS FOREIGN KEY (pais) REFERENCES PAIS(idPais)
);

-- DESENVOLUPA
CREATE TABLE DESENVOLUPA (
  desenvolupador     NUMBER NOT NULL,
  aplicacio          VARCHAR2(40) NOT NULL,
  CONSTRAINT PK_DESENVOLUPA PRIMARY KEY (desenvolupador, aplicacio),
  CONSTRAINT FK_DESENVOLUPA_DESENVOLUPADOR FOREIGN KEY (desenvolupador) REFERENCES DESENVOLUPADOR(idDesenvolupador),
  CONSTRAINT FK_DESENVOLUPA_APLICACIONS FOREIGN KEY (aplicacio) REFERENCES APLICACIONS(idAplicacio)
);

-- MODELS DISPOSITIU
CREATE TABLE MODELSDISPOSITIU (
  codiImei       NUMBER(15) NOT NULL,
  soperatiu      VARCHAR2(20) NOT NULL,
  modelDispositiu VARCHAR2(30) NOT NULL,
  resolucio      VARCHAR2(20) NOT NULL,
  usuari         NUMBER NOT NULL,
  CONSTRAINT PK_MODELSDISPOSITIU PRIMARY KEY (codiImei),
  CONSTRAINT FK_MODELSDISPOSITIU_USUARI FOREIGN KEY (usuari) REFERENCES USUARI(nmobil),
  CONSTRAINT FK_MODELSDISPOSITIU_SOPERATIU FOREIGN KEY (soperatiu) REFERENCES SOPERATIU(idSO)
);

-- DESCARREGUES
CREATE TABLE DESCARREGUES (
  aplicacio      VARCHAR2(40) NOT NULL,
  dispositiu     NUMBER(15) NOT NULL,
  usuari         NUMBER NOT NULL,
  pais           CHAR(2) NOT NULL,
  dataDesc       DATE NOT NULL,
  modePagament   VARCHAR2(22) NOT NULL,
  preu           NUMBER NOT NULL,
  operador       VARCHAR2(20) NOT NULL,
  CONSTRAINT PK_DESCARREGUES PRIMARY KEY (aplicacio, dispositiu),
  CONSTRAINT FK_DESCARREGUES_APLICACIO FOREIGN KEY (aplicacio) REFERENCES APLICACIONS(idAplicacio),
  CONSTRAINT FK_DESCARREGUES_MDISPOSITIU FOREIGN KEY (dispositiu) REFERENCES MODELSDISPOSITIU(codiImei),
  CONSTRAINT FK_DESCARREGUES_USUARI FOREIGN KEY (usuari) REFERENCES USUARI(nmobil), 
  CONSTRAINT FK_DESCARREGUES_PAIS FOREIGN KEY (pais) REFERENCES PAIS(idPais), 
  CONSTRAINT CK_DESCARREGUES CHECK (modePagament IN('paypal', 'targeta credit', 'transferencia bancaria'))
);
 
-- Estadstica 1
CREATE TABLE EST1_NUMTOTALDESCARREGUES (
  numDescarregues  NUMBER DEFAULT 0 NOT NULL, 
  CONSTRAINT PK_EST1_NUMTOTALDESCARREGUES PRIMARY KEY (numDescarregues)
);

-- Estadstica 2
CREATE TABLE EST2_TOTALEUROSDESC (
  totalEuros NUMBER DEFAULT 0 NOT NULL, 
  CONSTRAINT PK_EST2_TOTALEUROSDESC PRIMARY KEY (totalEuros)
);

-- Estadstica 3
CREATE TABLE EST3_NMIGAPLICUSUARI (
  anyDescarrega     NUMBER(4) NOT NULL,
  mitjanaAplicacions NUMBER DEFAULT 0 NOT NULL, 
  CONSTRAINT PK_EST3_NMIGAPLICUSUARI PRIMARY KEY (anyDescarrega)
);

-- Estadstica 4
CREATE TABLE EST4_MAXDESCARREGUES (
  anyDescarregues    NUMBER(4) NOT NULL,
  idDesenvolupador   NUMBER DEFAULT 0 NOT NULL,
  numDescarregues    NUMBER DEFAULT 0 NOT NULL, 
  CONSTRAINT PK_EST4_MAXIMDESCARREGUES PRIMARY KEY (anyDescarregues)
);

-- Estadstica 5
CREATE TABLE EST5_APLICMESRECAPTACIO (
  anyDescarrega      NUMBER(4) NOT NULL,
  idAplicacio        VARCHAR2(40) NOT NULL,
  idDesenvolupador   NUMBER DEFAULT 0 NOT NULL, 
  CONSTRAINT PK_EST5_APLICMESRECAPTACIO PRIMARY KEY (anyDescarrega)
);

-- Estadstica 6
CREATE TABLE EST6_NUMEROUSUARISDIFERENTS (
  anyDescarrega     NUMBER(4) NOT NULL,
  idPais            CHAR(2) NOT NULL,
  numUsuaris        NUMBER DEFAULT 0 NOT NULL, 
  CONSTRAINT PK_EST6_NUMEROUSUARISDIFERENTS PRIMARY KEY (anyDescarrega, idPais)
);

-- Estadstica 7
CREATE TABLE EST7_INGRESSOSUSUARISPAIS (
  anyDescarrega              NUMBER(4) NOT NULL,
  idPais                     CHAR(2) NOT NULL,
  ingressosUsuarisRegistrats NUMBER DEFAULT 0 NOT NULL, 
  CONSTRAINT PK_EST7_INGRESSOSUSUARISPAIS PRIMARY KEY (anyDescarrega, idPais)
);

-- Estadstica 8
CREATE TABLE EST8_APLICDIFDESC (
  anyDescarrega            NUMBER(4) NOT NULL,
  idPais                   CHAR(2) NOT NULL,
  numAplicDifDescarregades NUMBER DEFAULT 0 NOT NULL, 
  CONSTRAINT PK_EST8_APLICDIFDESC PRIMARY KEY (anyDescarrega, idPais)
);

-- LOG
CREATE TABLE LOG1 (
  codi           NUMBER NOT NULL,
  dat            VARCHAR2(25) NOT NULL,
  procediment    VARCHAR2(40) NOT NULL,
  paramEntrada   VARCHAR2(400) NOT NULL,
  paramSortida   VARCHAR2(400) NOT NULL, 
  CONSTRAINT PK_LOG1 PRIMARY KEY (codi)
);

CREATE SEQUENCE SQ_LOG1 INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER TR_BI_LOG1
  BEFORE INSERT ON LOG1
  FOR EACH ROW
BEGIN
  SELECT SQ_LOG1.NEXTVAL INTO :NEW.codi
  FROM DUAL;
END TR_BI_LOG1;
/




