create or replace PROCEDURE SP_ALTA_MODIF_EST5 (
   p_anyDescarrega IN EST5_APLICMESRECAPTACIO.anyDescarrega%TYPE
)
IS
/******************************************************************************
   Nom:                 SP_ALTA_MODIF_EST5

   Propsit:            Afegeix una entrada a la taula estadstica EST5_APLICMESRECAPTACIO. 
                        La informaci introduda correspon  a l'apliacacio que ms 
			diners ha recaptat aix com el seu desenvolupador.
                   
   Parmetres entrada:  -p_any: any en qu es produeix la descrrega

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
   
******************************************************************************/
paramEntrada        VARCHAR2(500);
RSP                 VARCHAR2 (500);
existeix            INTEGER;
p_aplicacio         NUMBER;
p_desenvolupador    NUMBER;
suma                NUMBER;
err_msg             VARCHAR2 (200);
E_NUL               EXCEPTION;

BEGIN
   paramEntrada:='p_anyDescarrega '||to_char(p_anyDescarrega);

  --verifiquem que el camp obligatori no sigui nul
   IF (p_anyDescarrega IS NULL) THEN 
     RAISE E_NUL;    
   END IF;

  -- trobem l'aplicaci que ms recapta en l'any indicat
   SELECT aplicacio, s
   INTO p_aplicacio, suma
   FROM (SELECT aplicacio, SUM(preu) AS s FROM DESCARREGUES WHERE EXTRACT(YEAR FROM dataDesc)=p_anyDescarrega GROUP BY aplicacio ORDER BY s DESC)
   WHERE rownum=1;

  -- trobem el desenvolupador de l'aplicacio
   SELECT desenvolupador
   INTO p_desenvolupador
   FROM DESENVOLUPA
   WHERE aplicacio=p_aplicacio;

  -- calculem si existeix la entrada en la taula EST5_APLICMESRECAPTACIO corresponent a l'any indicat
   SELECT COUNT (*)
   INTO existeix
   FROM EST5_APLICMESRECAPTACIO
   WHERE anyDescarrega=p_anyDescarrega;
   
  --Si no existeix l'entrada, l'afegim 
   IF (existeix=0) THEN 
     INSERT INTO EST5_APLICMESRECAPTACIO (anyDescarrega, idAplicacio, idDesenvolupador) 
     VALUES (p_anyDescarrega, p_aplicacio, p_desenvolupador);   
  
  --Si existeix l'actualitzem
   ELSE
     UPDATE EST5_APLICMESRECAPTACIO 
     SET idAplicacio=p_aplicacio,
         idDesenvolupador=p_desenvolupador
     WHERE anyDescarrega=p_anyDescarrega;
   END IF;  
 
  -- finalitza amb xito
   RSP:='OK';
   SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODIF_EST5', paramEntrada, 'RSP: '||RSP);

   COMMIT;
EXCEPTION

  -- no finalitza amb xito perque el valor obligatori es nul
   WHEN E_NUL THEN
        RSP:='ERROR EST5 el camp any no pot ser nulo'; 
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODIF_EST5', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finalitza amb xit per algn altre motiu        
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR EST5: '||err_msg;
         SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODIF_EST5', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;
        
END SP_ALTA_MODIF_EST5;
