create or replace PROCEDURE SP_ALTA_MODIF_EST6 (
   p_anyDescarrega IN EST6_NUMEROUSUARISDIFERENTS.anyDescarrega%TYPE,
   p_idPais IN EST6_NUMEROUSUARISDIFERENTS.idPais%TYPE
)
IS
/******************************************************************************
   Nom:                 SP_ALTA_MODIF_EST6

   Propsito:           Afegeix una entrada a la taula estadstica EST6_NUMEROUSUARISDIFERENT. 
                       La informaci introduda correspon  al nmero dusuaris diferents que han
                       fet coma mnim una descrrega en un pas i any concret.
                   
   Parmetres entrada:    -p_idPais: identificador del pais
                          -p_any: any en qu es produeix la descrrega

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat  
   
******************************************************************************/
paramEntrada      VARCHAR2(500);
RSP               VARCHAR2 (500);
existeix          INTEGER;
p_numUsuaris      NUMBER;
err_msg           VARCHAR2 (200);
camp_nul          VARCHAR2(20);
E_NUL             EXCEPTION;

BEGIN

   paramEntrada:='p_anyDescarrega: '||to_char(p_anyDescarrega)||
		 ', p_idPais: '||p_idPais;

  --verifiquem que els camps obligatoris no siguin nuls
   IF (p_anyDescarrega IS NULL) THEN 
     camp_nul:='anyDescarrega';
     RAISE E_NUL;
   ELSIF (p_idPais IS NULL) THEN 
     camp_nul:='idPais';
     RAISE E_NUL;  
   END IF;

  -- calculem el nmero d'usuaris que han realizat almenys una descrrega en el pas i any indicat
   SELECT COUNT(DISTINCT usuari)
   INTO p_numUsuaris
   FROM DESCARREGUES d
   WHERE EXTRACT(YEAR FROM d.dataDesc)=p_anyDescarrega AND pais=p_idPais;

  -- calculem si existeix l'entrada en la taula EST6_NUMEROUSUARIDIFERENT corresponent a l'any indicat
   SELECT COUNT (*)
   INTO existeix
   FROM EST6_NUMEROUSUARISDIFERENTS
   WHERE anyDescarrega=p_anyDescarrega AND idPais=p_idPais;   

  --Si no existeix l'entrada, l'afegim 

   IF (existeix=0) THEN 
     INSERT INTO EST6_NUMEROUSUARISDIFERENTS (anyDescarrega, idPais, numUsuaris) 
     VALUES (p_anyDescarrega, p_idPais, p_numUsuaris);    
 
  --Si existeix l'actualitzem
   ELSE
     UPDATE EST6_NUMEROUSUARISDIFERENTS 
     SET numUsuaris=p_numUsuaris
     WHERE anyDescarrega=p_anyDescarrega AND idPais=p_idPais;
   END IF;  
 
  -- finalitza amb xit
   RSP:='OK';
   SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODIF_EST6', paramEntrada, 'RSP: '||RSP);

   COMMIT;
EXCEPTION

  -- no finalitza amb xito perque el valor obligatori es nul
   WHEN E_NUL THEN
        RSP:='ERROR EST6 el camp any no pot ser nulo'; 
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODIF_EST6', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;

  -- no finalitza amb xit per algn altre motiu        
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR EST6: '||err_msg;
         SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODIF_EST6', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;
        
END SP_ALTA_MODIF_EST6;
