create or replace PROCEDURE SP_ALTA_MODIF_EST8 (
   p_anyDescarrega IN EST8_APLICDIFDESC.anyDescarrega%TYPE,
   p_idPais IN EST8_APLICDIFDESC.idPais%TYPE
)
IS
/******************************************************************************
   Nom:                 SP_ALTA_MODIF_EST8

   
   Propsit:            Actualitza una entrada a la taula estadstica EST8_APLICDIFDESC
                        La informaci introduda correspon al nmero daplicacions diferents 
                       descarregades en un pas i any concret.

               
   Parmetres entrada:    -p_idPais: identificador del pas
                          -p_anyDEscarrega: any en qu es produeix la descrrega
   
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat

******************************************************************************/
paramEntrada                VARCHAR2(500);
RSP                         VARCHAR2 (500);
existeix                    INTEGER;
p_numAplicDifDescarregades  NUMBER;
err_msg                     VARCHAR2 (200);
camp_nul                    VARCHAR2(20);
E_NUL                       EXCEPTION;

BEGIN

   paramEntrada:='p_anyDescarrega: '||to_char(p_anyDescarrega)||', p_idPais: '||p_idPais;

  --verifiquem que els camps obligatoris no siguin nuls
   IF (p_anyDescarrega IS NULL) THEN 
     camp_nul:='anyDescarrega';
     RAISE E_NUL;
   ELSIF (p_idPais IS NULL) THEN 
     camp_nul:='idPais';
     RAISE E_NUL;  
   END IF;

  -- calculem els ingressos generata en el pais i l'any indicats
   SELECT COUNT(DISTINCT aplicacio)
   INTO p_numAplicDifDescarregades
   FROM DESCARREGUES d
   WHERE EXTRACT(YEAR FROM d.dataDesc)=p_anyDescarrega AND pais=p_idPais;

  -- calculem si existeix l'entrada a la taula EST8_APLICDIFDESC corresponent a l'any indicat
   SELECT COUNT (*)
   INTO existeix
   FROM EST8_APLICDIFDESC
   WHERE anyDescarrega=p_anyDescarrega AND idPais=p_idPais;   
  
  --Si no existeix l'entrada, l'afegim

   IF (existeix=0) THEN 
     INSERT INTO EST8_APLICDIFDESC (anyDescarrega, idPais, numAplicDifDescarregades) 
     VALUES (p_anyDescarrega, p_idPais, p_numAplicDifDescarregades);  
   
  --Si existeix l'actualitzem
   ELSE
     UPDATE EST8_APLICDIFDESC 
     SET numAplicDifDescarregades=p_numAplicDifDescarregades
     WHERE anyDescarrega=p_anyDescarrega AND idPais=p_idPais;
   END IF;  
 
  -- finalitza amb xit
   RSP:='OK';
   SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODIF_EST8', paramEntrada, 'RSP: '||RSP);

   COMMIT;
EXCEPTION
    -- no finalitza amb xit perque el valor obligatori es nul
   WHEN E_NUL THEN
        RSP:='ERROR EST8 el camp any no pot ser nulo'; 
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODIF_EST8', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;

  -- no finalitza amb xit per algn altre motiu        
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR EST8 '||err_msg;
         SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODIF_EST8', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;
        
END SP_ALTA_MODIF_EST8;
