create or replace PROCEDURE SP_ALTA_LOG1 (
   p_dat            IN LOG1.dat%TYPE,
   p_procediment    IN LOG1.procediment%TYPE,
   p_paramEntrada   IN LOG1.paramEntrada%TYPE,
   p_paramSortida   IN LOG1.paramsortida%TYPE
)
IS
/******************************************************************************
   Nom:                SP_ALTA_LOG1

   Propsit:           Afegeix una entrada la la taula LOG1
                   
   Parmetres entrada:  p_dat, p_procediment, p_paramEntrada, p_paramSortida            

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus de error' si ha fracasat
******************************************************************************/

paramEntrada  VARCHAR2(500);
RSP           VARCHAR2 (500);
err_msg       VARCHAR2 (200);
camp_nul       VARCHAR2(20);
E_NUL          EXCEPTION;

  
PRAGMA AUTONOMOUS_TRANSACTION;

BEGIN

   paramEntrada:='p_dat: '||p_dat||
		 ',p_procediment: '||p_procediment||
		 ',p_paramEntrada: '||p_paramEntrada||
                ', p_paramSortida: '||p_paramSortida;

  -- verifiquem que els camps obligatoris no siguin nuls
   
  IF (p_dat IS NULL) THEN 
     camp_nul:='dat';
     RAISE E_NUL;
   ELSIF (p_procediment IS NULL) THEN 
     camp_nul:='procediment';     
     RAISE E_NUL;
   ELSIF (p_paramEntrada IS NULL) THEN 
     camp_nul:='paramEntrada';
     RAISE E_NUL;     
   ELSIF (p_paramSortida IS NULL) THEN 
     camp_nul:='paramSortida';
     RAISE E_NUL;   
   END IF;
 

  -- Insertem la nova entrada a la taula log1

   INSERT INTO LOG1 (dat, procediment, paramEntrada, paramSortida)
   VALUES (p_dat, p_procediment, p_paramEntrada, p_paramSortida);
   
  -- finaliza amb xit
   RSP:='OK';


   COMMIT;
EXCEPTION
     WHEN E_NUL THEN
        RSP:='ERROR LOG: el camp '||camp_nul||' no pot ser nul'; 
        INSERT INTO LOG1 (dat, procediment, paramEntrada, paramSortida)
        VALUES (to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_LOG1', p_paramEntrada, 'RSP: '||RSP);
        ROLLBACK;

       
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR LOG: '||err_msg;
        INSERT INTO LOG1 (dat, procediment, paramEntrada, paramSortida)
        VALUES (to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_LOG1', p_paramEntrada, 'RSP: '||RSP);
        ROLLBACK;
              
END SP_ALTA_LOG1;