create or replace PROCEDURE SP_ALTA_MODIF_EST3 (
   p_anyDescarrega IN EST3_NMIGAPLICUSUARI.anyDescarrega%TYPE
)
IS
/******************************************************************************
   Nombre:              SP_ALTA_MODIF_EST3

  Propsit:             Actualitza una entrada a la taula estadstica
                        EST3_NMIGAPLICSUSUARI La informaci introduda correspon  
                        al numero mig daplicacions descarregades per un usuari.
                   
   Parmetres entrada:  p_anyDescarrega: any en qu es produeix la descrrega

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat                 
   
******************************************************************************/
paramEntrada     VARCHAR2(500);
RSP              VARCHAR2 (500);
p_mitjanaAplicacions    NUMBER;
descarregues            NUMBER;
usuaris                 NUMBER;
existeix                INTEGER;
err_msg                 VARCHAR2 (200);
E_NUL                   EXCEPTION;

BEGIN

   paramEntrada:='p_anyDescarrega: '||to_char(p_anyDescarrega);

  --verifiquem que el camp obligatori no sigui nul
   IF (p_anyDescarrega IS NULL) THEN 
     RAISE E_NUL;    
   END IF;

  -- calculem la mitjana de descrregues realitzades en l'any indicat
   SELECT COUNT (*)
   INTO descarregues
   FROM DESCARREGUES
   WHERE EXTRACT(YEAR FROM dataDesc)=p_anyDescarrega;
   SELECT COUNT (*)
   INTO usuaris
   FROM USUARI;
   p_mitjanaAplicacions:=descarregues/usuaris;

  -- calculem si existeix l'entrada en la tauala EST3_NMIGPALICUSUARI corresponent a l'any indicat
   SELECT COUNT (*)
   INTO existeix
   FROM EST3_NMIGAPLICUSUARI
   WHERE anyDescarrega=p_anyDescarrega; 
  
  --Si no existe l'entrada, l'afegim 
   IF (existeix=0) THEN 
     INSERT INTO EST3_NMIGAPLICUSUARI (anyDescarrega, mitjanaAplicacions) VALUES (p_anyDescarrega, p_mitjanaAplicacions);     
  
  --Si existe l'actualitzem
   ELSE
     UPDATE EST3_NMIGAPLICUSUARI 
     SET mitjanaAplicacions=p_mitjanaAplicacions
     WHERE anyDescarrega=p_anyDescarrega;
   END IF;  
 
  -- finalitza amb xit
   RSP:='OK';
   SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODIF_EST3', paramEntrada, 'RSP: '||RSP);

   COMMIT;

EXCEPTION
  -- no finalitza amb xito perque el valor obligatori es nul
   WHEN E_NUL THEN
        RSP:='ERROR EST3 el campo anyDescarrega no pot ser nul'; 
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODIF_EST3', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;

  -- no finalitza amb xito per algn altre motiu         
   WHEN OTHERS THEN
        err_msg:=SUBSTR(SQLERRM, 1, 200);
        RSP:='ERROR EST3: '||err_msg;
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODIF_EST3', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;
              
END SP_ALTA_MODIF_EST3;