create or replace PROCEDURE SP_ALTA_MODIF_EST4 (
   p_anyDescarregues IN EST4_MAXDESCARREGUES.anyDescarregues%TYPE
)
IS
/******************************************************************************
   Nom:                 SP_ALTA_MODIF_EST4

   Propsit:            Afegeix una entrada a la taula estadstica EST4_MAXIMDESCARREGUES.
                        La informaci introduda correspon  al desenvolupador que 
                        tingui el mxim nmero de descrregues.
                   
   Parmetres entrada:    -p_anyDescarrega: any en qu es produeix la descrrega


   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat

******************************************************************************/
paramEntrada     VARCHAR2(500);
RSP              VARCHAR2 (500);
existeix         INTEGER;
p_mitjaDescarregues  NUMBER;
p_idDesenvolupador   NUMBER;
p_numDescarregues    NUMBER;
num INTEGER;
err_msg VARCHAR2 (200);
E_NUL EXCEPTION;

BEGIN

   paramEntrada:='p_anyDescarregues: '||to_char(p_anyDescarregues);

  --verifiquem que el campo obligatori no sigui nul
   IF (p_anyDescarregues IS NULL) THEN 
     RAISE E_NUL;    
   END IF;

  -- calculem el desenvolupador amb el mxim nombre de descarregues en l'any indicat
  
   SELECT desenvolupador,  numDescarregues 
   INTO p_idDesenvolupador, p_numDescarregues
   FROM (SELECT d.desenvolupador AS desenvolupador, COUNT(des.dispositiu) AS numDescarregues FROM DESENVOLUPA d JOIN DESCARREGUES des ON d.aplicacio=des.aplicacio 
        WHERE EXTRACT(YEAR FROM des.dataDesc)=p_anyDescarregues GROUP BY d.desenvolupador ORDER BY numDEscarregues DESC)
   WHERE rownum=1;
   
  -- calculem si existeix l'entrada en la taula EST4_MAXIMDESCARREGUES corresponent a l'any indicat
   SELECT COUNT (*)
   INTO existeix
   FROM EST4_MAXDESCARREGUES
   WHERE anyDescarregues=p_anyDescarregues; 
  
  --Si no existeix l' entrada, l'afegim 
     IF (existeix=0) THEN 
     INSERT INTO EST4_MAXDESCARREGUES (anyDescarregues, idDesenvolupador, numDescarregues) 
     VALUES (p_anyDescarregues, p_idDesenvolupador, p_numDescarregues);     
  
  --Si existe l'actualitzem
   ELSE
     UPDATE EST4_MAXDESCARREGUES 
     SET idDesenvolupador=p_idDesenvolupador,
         numDescarregues=p_numDescarregues
     WHERE anyDescarregues=p_anyDescarregues;
   END IF;  
 
  -- finalitza amb xito
   RSP:='OK';
   SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODIF_EST4', paramEntrada, 'RSP: '||RSP);

   COMMIT;
EXCEPTION
  -- no finalitza amb xito perque el valor obligatori es nulo
   WHEN E_NUL THEN
        RSP:='ERROR EST4 el camp anyDescarrega no pot ser nul'; 
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODIF_EST4', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;

  -- no finalitza amb xito per algn altre motiu         
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR EST4: '||err_msg;
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODIF_EST4', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;
              
END SP_ALTA_MODIF_EST4;