create or replace PROCEDURE SP_ALTA_DESENVOLUPADORS (
  
   p_idDesenvolupador         IN  DESENVOLUPADOR.idDesenvolupador%TYPE,
   p_nom         IN  DESENVOLUPADOR.nom%TYPE,
   p_nomRepresentant      IN  DESENVOLUPADOR.nomRepresentant%TYPE,
   p_pais      IN  DESENVOLUPADOR.pais%TYPE, 
   p_adreca IN  DESENVOLUPADOR.adreca%TYPE,
   p_telefon IN DESENVOLUPADOR.telefon%TYPE,
   RSP              OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_ALTA_DESENVOLUPADOR
   Propsit:            afegeix un desenvoplupador a la taula DESENVOPLUPADORS
                   
   Parmetres entrada:  p_idDesenvolupadors, p_nom,p_nomRepresentant,p_pais,
                         p_adreca, p_telefon                    
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
paramEntrada  VARCHAR2(500);
existeix INTEGER;
err_msg  VARCHAR2 (200);
camp_nul VARCHAR2(50);
E_NUL    EXCEPTION;
E_FK3     EXCEPTION;
BEGIN

   paramEntrada:= ', p_idDesenvolupador: '||p_idDesenvolupador||
                ', p_nom: '||p_nom||
                 ', p_nomRepresentant: '||p_nomRepresentant||
                 ', p_pais: '||p_pais||
                 ', p_adreca: '||p_adreca||  
                 ', p_telefon: '||p_telefon;

  --verifica que els camps obligatoris no siguin nuls
    IF (p_idDesenvolupador IS NULL) THEN 
     camp_nul:= 'idDesenvolupador';
     RAISE E_NUL;
     
   ELSIF (p_nom IS NULL) THEN 
     camp_nul:= 'nom';
     RAISE E_NUL;
   ELSIF (p_nomRepresentant IS NULL) THEN 
     camp_nul:= 'nomRepresentant';
     RAISE E_NUL;
   ELSIF (p_adreca IS NULL) THEN 
     camp_nul:= 'adreca';
     RAISE E_NUL;
   ELSIF (p_telefon IS NULL) THEN 
     camp_nul:= 'telefon';
     RAISE E_NUL;
   END IF; 

  
   -- inserta el nou desenvoplupador
   INSERT INTO DESENVOLUPADOR ( idDesenvolupador, nom, nomRepresentant, pais, adreca, telefon)
   VALUES ( p_idDesenvolupador, p_nom, p_nomRepresentant, p_pais, p_adreca, p_telefon);
   
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESENVOLUPADORS', paramEntrada, 'RSP: '||RSP);
   COMMIT;
   
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESENVOLUPADORS', paramEntrada, 'RSP: '||RSP);
        
   WHEN E_FK3 THEN
        RSP:= 'ERROR: el pais no existeix';        
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESENVOLUPADORS', paramEntrada, 'RSP: '||RSP);
        
   
              
END SP_ALTA_DESENVOLUPADORS;