create or replace PROCEDURE SP_BAIXA_DESENVOLUPADORS (
   p_idDesenvolupador IN  DESENVOLUPADOR.idDesenvolupador%TYPE,
   RSP    OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_BAIXA_DESENVOLUPADORS
   Propsit:            elimina un model de la taula DESENVOLUPADORS
                   
   Parmetres entrada:  id_Desenvolupadors
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
paramEntrada  VARCHAR2(500);
existeix      INTEGER;
err_msg       VARCHAR2(200);
E_NUL         EXCEPTION;
E_NO_EXISTEIX EXCEPTION;
BEGIN

   paramEntrada:= 'id_Desenvolupador: '||p_idDesenvolupador;
     
  --verifica que els camps obligatoris no siguin nuls
   IF (p_idDesenvolupador IS NULL) THEN 
     RAISE E_NUL;
   END IF; 

  --verifica que existeix el desenvolupador
   SELECT COUNT (*)
   INTO existeix
   FROM DESENVOLUPADOR
   WHERE idDesenvolupador = p_idDesenvolupador;

   IF (existeix = 0) THEN
     RAISE E_NO_EXISTEIX;
   END IF;

  -- elimina el desenvolupador
   DELETE FROM DESENVOLUPADOR
   WHERE idDesenvolupador = p_idDesenvolupador;

  -- finalitza amb xit   
   RSP:= 'OK';
   SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_DESENVOLUPADORS', paramEntrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp idDesenvolupadors no pot ser nul';
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_DESENVOLUPADORS', paramEntrada, 'RSP: '||RSP);
    
   WHEN E_NO_EXISTEIX THEN
        RSP:= 'ERROR: el desenvolupador no existeix';
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_DESENVOLUPADORS', paramEntrada, 'RSP: '||RSP);
    
   WHEN OTHERS THEN
        ROLLBACK;
        IF (SQLCODE = -02292) THEN 
          RSP:= 'ERROR: no es pot esborrar el desenvolupador perqu est relacionat amb alguna aplicaci existent.';
          SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_DESENVOLUPADORS', paramEntrada, 'RSP: '||RSP);
        
        ELSE
          err_msg := SUBSTR (SQLERRM, 1, 200);
          RSP:=    'ERROR: ' || err_msg;
          SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_DESENVOLUPADORS', paramEntrada, 'RSP: '||RSP);
        END IF;        

              
END SP_BAIXA_DESENVOLUPADORS;