create or replace PROCEDURE SP_MODIFICACIO_DESENVOLUPADORS (
   p_idDesenvolupador       IN  DESENVOLUPADOR.idDesenvolupador%TYPE,
   p_nom IN  DESENVOLUPADOR.nom%TYPE,
   p_nomRepresentant  IN  DESENVOLUPADOR.nomRepresentant%TYPE,
   p_pais IN  DESENVOLUPADOR.pais%TYPE,
   p_adreca  IN  DESENVOLUPADOR.adreca%TYPE,
   p_telefon IN  DESENVOLUPADOR.telefon%TYPE,
   RSP          OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_MODIFICACIO_DESENVOLUPADORS
   Propsit:            modifica dades d'un desenvolupador a la taula DESENVOLUPADORS 
                   
   Parmetres entrada:  p_idDesenvolupador, p_nom, p_nomRepresentant, p_pais, p_adreca,
                        p_telefon
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
paramEntrada  VARCHAR2(500);
existeix          INTEGER;
err_msg           VARCHAR2(200);
camp_nul          VARCHAR2(20);
E_NUL             EXCEPTION;
E_NO_EXISTEIX     EXCEPTION;
BEGIN
   
   paramEntrada:= 'p_idDesenvolupador: '||p_idDesenvolupador||
                 ', p_nom: '||p_nom||
                 ', p_nomRepresentant: '||p_nomRepresentant||
                 ', p_pais: '||p_pais||
                 ', p_adreca: '||p_adreca||
                 ', p_telefon: '||p_telefon;
   
  --verifica que els camps obligatoris no siguin nuls
   IF (p_idDesenvolupador IS NULL) THEN 
     camp_nul:= 'idDesenvolupador';
     RAISE E_NUL;
   ELSIF (p_nom IS NULL) THEN 
     camp_nul:= 'nom';
     RAISE E_NUL;
   ELSIF (p_nomRepresentant IS NULL) THEN 
     camp_nul:= 'nomRepresentant';
     RAISE E_NUL;
   ELSIF (p_pais IS NULL) THEN 
     camp_nul:= 'pais';
     RAISE E_NUL;
   ELSIF (p_adreca IS NULL) THEN 
     camp_nul:= 'adreca';
     RAISE E_NUL;
   ELSIF (p_telefon IS NULL) THEN 
     camp_nul:= 'telefon';
     RAISE E_NUL;
   END IF; 

  --verifica que el desenvolupador existeix
   SELECT COUNT (*)
   INTO existeix
   FROM DESENVOLUPADOR
   WHERE idDesenvolupador = p_idDesenvolupador ;

   IF (existeix = 0) THEN
     RAISE E_NO_EXISTEIX;
   END IF;

   -- modifica les dades del desenvolupador
   UPDATE DESENVOLUPADOR 
   SET nom = p_nom, 
       nomRepresentant = p_nomRepresentant,
       pais = p_pais,
       adreca= p_adreca,
       telefon = p_telefon
   WHERE idDesenvolupador = p_idDesenvolupador;
     
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_DESENVOLUPADORS', paramEntrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_DESENVOLUPADORS', paramEntrada, 'RSP: '||RSP);
            
   WHEN E_NO_EXISTEIX THEN
        RSP:= 'ERROR: el desenvolupador no existeix';
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_DESENVOLUPADORS', paramEntrada, 'RSP: '||RSP);
            
   
   WHEN OTHERS THEN
        ROLLBACK;
        
        IF (SQLCODE = -00001) THEN 
          RSP:= 'ERROR: ja existeix un desenvolupador amb mateix: codi identificactiu';
          SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_DESENVOLUPADORS', paramEntrada, 'RSP: '||RSP);
          
        ELSE 
          err_msg := SUBSTR (SQLERRM, 1, 200);
          RSP:=    'ERROR: ' || err_msg;
          SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_DESENVOLUPADORS', paramEntrada, 'RSP: '||RSP);
          
        END IF;
              
END SP_MODIFICACIO_DESENVOLUPADORS;