create or replace PROCEDURE SP_ALTA_USUARIS (
   p_nmobil                 IN  USUARI.nmobil%TYPE,
   p_operador       IN  USUARI.operador%TYPE,
   p_pais IN  USUARI.pais%TYPE,
   p_correu IN  USUARI.correu%TYPE,
  
   RSP                    OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_USUARIS
   Propsit:            afegeix un USUARI a la taula USUARI 
                   
   Parmetres entrada:  p_nmobil, p_operador, p_pais, p_correu,            
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
paramEntrada  VARCHAR2(500);
existeix INTEGER;
duplicat INTEGER;
err_msg  VARCHAR2 (200);
camp_nul VARCHAR2(20);
E_NUL    EXCEPTION;
E_FK     EXCEPTION;
E_FK1    EXCEPTION;
E_DUP    EXCEPTION;
BEGIN
   
   paramEntrada:= 'p_nmobil: '||p_nmobil||
                 ', p_operador: '||p_operador||
                 ', p_correu: '||p_correu||
		 ', p_pais: '||p_pais;
   
  --verifica que els camps obligatoris no siguin nuls
   IF (p_nmobil IS NULL) THEN 
     camp_nul:= 'nmobil';
     RAISE E_NUL;
   ELSIF (p_correu IS NULL) THEN 
     camp_nul:= 'correu';
     RAISE E_NUL;
   END IF; 

  --verifica que el pais existeix
   SELECT COUNT (*)
   INTO existeix
   FROM PAIS
   WHERE idPais = p_pais ;

   IF (existeix = 0) THEN
     RAISE E_FK;
   END IF;

 
    -- inserta el nou USUARIS
   INSERT INTO USUARI (nmobil, operador, correu, pais)
   VALUES (p_nmobil, p_operador, p_correu, p_pais);
   
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY'), 'SP_ALTA_USUARIS', paramEntrada, 'RSP: '||RSP);
   COMMIT;
   
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY'), 'SP_ALTA_USUARIS', paramEntrada, 'RSP: '||RSP);
        
   WHEN E_FK THEN
        RSP:= 'ERROR: el pais no existeix';        
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY'), 'SP_ALTA_USUARIS', paramEntrada, 'RSP: '||RSP);
        
     WHEN E_FK1 THEN
        RSP:= 'ERROR: el operador no existeix';        
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY'), 'SP_ALTA_USUARIS', paramEntrada, 'RSP: '||RSP);

                
END SP_ALTA_USUARIS;