create or replace PROCEDURE SP_BAIXA_USUARIS (
   p_nmobil IN  USUARI.nmobil%TYPE,
   RSP    OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_USUARI
   Propsit:            elimina un model de la taula USUARI
                   
   Parmetres entrada:  nmobil
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
paramEntrada  VARCHAR2(500);
existeix      INTEGER;
err_msg       VARCHAR2(200);
E_NUL         EXCEPTION;
E_NO_EXISTEIX EXCEPTION;
BEGIN

   paramEntrada:= 'nmobil: '||p_nmobil;
     
  --verifica que els camps obligatoris no siguin nuls
   IF (p_nmobil IS NULL) THEN 
     RAISE E_NUL;
   END IF; 

  --verifica que existeix el usuari
   SELECT COUNT (*)
   INTO existeix
   FROM USUARI
   WHERE nmobil = p_nmobil;

   IF (existeix = 0) THEN
     RAISE E_NO_EXISTEIX;
   END IF;

  -- elimina el usuari
   DELETE FROM USUARI
   WHERE nmobil = p_nmobil;

  -- finalitza amb xit   
   RSP:= 'OK';
   SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_USUARIS', paramEntrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp nmobil no pot ser nul';
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_USUARIS', paramEntrada, 'RSP: '||RSP);
    
   WHEN E_NO_EXISTEIX THEN
        RSP:= 'ERROR: usuari no existeix';
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_USUARIS', paramEntrada, 'RSP: '||RSP);
    
   WHEN OTHERS THEN
        ROLLBACK;
        IF (SQLCODE = -02292) THEN 
          RSP:= 'ERROR: no es pot esborrar usuari perqu est relacionat amb alguna aplicaci existent.';
          SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_USUARIS', paramEntrada, 'RSP: '||RSP);
        
        ELSE
          err_msg := SUBSTR (SQLERRM, 1, 200);
          RSP:=    'ERROR: ' || err_msg;
          SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_USUARIS', paramEntrada, 'RSP: '||RSP);
        END IF;        

              
END SP_BAIXA_USUARIS;