create or replace PROCEDURE SP_MODIFICACIO_USUARIS(
   p_nmobil      IN  USUARI.nmobil%TYPE,
   p_operador IN  USUARI.operador%TYPE,
   p_pais IN  USUARI.pais%TYPE,
   p_correu IN  USUARI.correu%TYPE,
   RSP          OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_MODIFICACIO_USUARIS
   Propsit:            modifica dades d'un usuari a la taula USUARI
                   
   Parmetres entrada:  p_nmobil, p_operador, p_pais, p_correu
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
paramEntrada  VARCHAR2(500);
existeix          INTEGER;
err_msg           VARCHAR2(200);
camp_nul          VARCHAR2(20);
E_NUL             EXCEPTION;
E_FK              EXCEPTION;
E_NO_EXISTEIX     EXCEPTION;
BEGIN
   
     paramEntrada:= 'p_nmobil: '||p_nmobil||
                 ', p_operador: '||p_operador||
                 ', p_pais: '||p_pais||
		 ', p_correu: '||p_correu;
   
  --verifica que els camps obligatoris no siguin nuls
   IF (p_nmobil IS NULL) THEN 
     camp_nul:= 'nmobil';
     RAISE E_NUL;
   ELSIF (p_correu IS NULL) THEN 
     camp_nul:= 'correu';
     RAISE E_NUL;
   END IF; 

  --verifica que el pais existeix
   SELECT COUNT (*)
   INTO existeix
   FROM PAIS
   WHERE idPais = p_pais ;

   IF (existeix = 0) THEN
     RAISE E_FK;
   END IF;

  
  
   -- modifica les dades de l'usuari
   UPDATE USUARI 
   SET nmobil=p_nmobil, 
       operador = p_operador,
       pais = p_pais,
       correu= p_correu
  
       
   WHERE nmobil = p_nmobil;
     
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_USUARIS', paramEntrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_USUARIS', paramEntrada, 'RSP: '||RSP);
            
   WHEN E_NO_EXISTEIX THEN
        RSP:= 'ERROR: el desenvolupador no existeix';
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_USUARIS', paramEntrada, 'RSP: '||RSP);
            
   
   WHEN OTHERS THEN
        ROLLBACK;
        
        IF (SQLCODE = -00001) THEN 
          RSP:= 'ERROR: ja existeix un usuari amb mateix: nmero de mbil';
          SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_USUARIS', paramEntrada, 'RSP: '||RSP);
          
        ELSE 
          err_msg := SUBSTR (SQLERRM, 1, 200);
          RSP:=    'ERROR: ' || err_msg;
          SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_USUARIS', paramEntrada, 'RSP: '||RSP);
          
        END IF;
              
END SP_MODIFICACIO_USUARIS;