create or replace PROCEDURE SP_ALTA_APLICACIONS (
   p_idAplicacio IN  APLICACIONS.idAplicacio%TYPE,
   p_versio         IN  APLICACIONS.versio%TYPE,
   p_dataPujada      IN  APLICACIONS.dataPujada%TYPE,
   p_URL IN APLICACIONS.URL%TYPE,
   p_resolucio IN APLICACIONS.resolucio%TYPE,
   p_activa IN APLICACIONS.activa%TYPE,

   RSP              OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_ALTA_APLICACIONS
   Propsit:            afegeix una aplicacio a la taula APLICACIONS
                   
   Parmetres entrada:  p_idAplicacio ,p_versio , p_data , p_desenvolupadors , p_sistemaOperatiu, p_enllacBinari,
   p_mida, p_descripcio, p_URL,p_preu, p_resolucio,p_activa 
                 
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
paramEntrada  VARCHAR2(500);
existeix INTEGER;
err_msg  VARCHAR2 (200);
camp_nul VARCHAR2(20);
E_NUL    EXCEPTION;
E_JA_EXISTEIX EXCEPTION;
E_FK     EXCEPTION;
E_FK1  EXCEPTION;
BEGIN

   paramEntrada:= 'p_idAplicacio: '||p_idAplicacio||
                 ', p_versio: '||p_versio||
                 ', p_dataPujada: '||p_dataPujada||
                 ', p_URL: '||p_URL||
                 ', p_resolucio: '||p_resolucio||
                 ', p_activa: '||p_activa;

  --verifica que els camps obligatoris no siguin nuls
   IF (p_idAplicacio IS NULL) THEN 
     camp_nul:= 'idAplicacio';
     RAISE E_NUL;
   ELSIF (p_versio IS NULL) THEN 
     camp_nul:= 'versio';
     RAISE E_NUL;
   ELSIF (p_dataPujada IS NULL) THEN 
     camp_nul:= 'dataPujada';
     RAISE E_NUL;
   ELSIF (p_URL IS NULL) THEN 
     camp_nul:= 'URL';
     RAISE E_NUL;
  ELSIF (p_resolucio IS NULL) THEN 
     camp_nul:= 'resolucio';
     RAISE E_NUL;
  ELSIF (p_activa IS NULL) THEN 
     camp_nul:= 'activa';
     RAISE E_NUL;

  END IF; 


 --verifiquem si existeix l'aplicacio
   SELECT COUNT (*)
   INTO existeix
   FROM APLICACIONS
   WHERE idAplicacio=p_idAplicacio;

   IF (existeix > 0) THEN
     RAISE E_JA_EXISTEIX;
   END IF; 

   -- inserta la nova aplicaci
   INSERT INTO APLICACIONS (idAplicacio, versio, dataPujada, URL,  resolucio, activa)
   VALUES (p_idAplicacio, p_versio, p_dataPujada, p_URL,  p_resolucio, p_activa);
   
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_APLICACIONS', paramEntrada, 'RSP: '||RSP);
   COMMIT;
   
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_APLICACIONS', paramEntrada, 'RSP: '||RSP);
       
  WHEN E_JA_EXISTEIX THEN
        RSP:= 'ERROR: la aplicacio ja existeix'; 
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_APLICACIONS', paramEntrada, 'RSP: '||RSP);   
        
   WHEN E_FK THEN
        RSP:= 'ERROR: el desenvolupador no existeix';        
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_APLICACIONS', paramEntrada, 'RSP: '||RSP);
        
   WHEN E_FK1 THEN
        RSP:= 'ERROR: el sistema operatiu no existeix';        
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_APLICACIONS', paramEntrada, 'RSP: '||RSP);
              
END SP_ALTA_APLICACIONS;