create or replace PROCEDURE SP_BAIXA_APLICACIONS (
   p_idAplicacio IN  APLICACIONS.idAplicacio%TYPE,
   RSP    OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_BAIXA_APLICACIONS
   Propsit:            elimina una aplicacio de la taula APLICACIONS
                   
   Parmetres entrada:  id_APLICACIONS
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
paramEntrada  VARCHAR2(500);
existeix      INTEGER;
err_msg       VARCHAR2(200);
E_NUL         EXCEPTION;
E_NO_EXISTEIX EXCEPTION;
BEGIN

   paramEntrada:= 'idAplicacio: '||p_idAplicacio;
     
  --verifica que els camps obligatoris no siguin nuls
   IF (p_idAplicacio IS NULL) THEN 
     RAISE E_NUL;
   END IF; 

  --verifica que existeix l'aplicacio
   SELECT COUNT (*)
   INTO existeix
   FROM APLICACIONS
   WHERE idAplicacio = p_idAplicacio;

   IF (existeix = 0) THEN
     RAISE E_NO_EXISTEIX;
   END IF;

  -- elimina l'aplicacio
   DELETE FROM APLICACIONS
   WHERE idAplicacio = p_idAplicacio;

  -- finalitza amb xit   
   RSP:= 'OK';
   SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_APLICACIONS', paramEntrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp idAplicacio no pot ser nul';
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_APLICACIONS', paramEntrada, 'RSP: '||RSP);
    
   WHEN E_NO_EXISTEIX THEN
        RSP:= 'ERROR: aplicacio no existeix';
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_APLICACIONS', paramEntrada, 'RSP: '||RSP);
     
   WHEN OTHERS THEN
        ROLLBACK;
        IF (SQLCODE = -02292) THEN 
          RSP:= 'ERROR: no es pot esborrar aplicacio.';
          SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_APLICACIONS', paramEntrada, 'RSP: '||RSP);
        
    
          
        ELSE
          err_msg := SUBSTR (SQLERRM, 1, 200);
          RSP:=    'ERROR: ' || err_msg;
          SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_APLICACIONS', paramEntrada, 'RSP: '||RSP);
        END IF;        

              
END SP_BAIXA_APLICACIONS;