create or replace PROCEDURE SP_MODIFICACIO_APLICACIONS (
   p_idAplicacio IN  APLICACIONS.idAplicacio%TYPE,
   p_versio         IN  APLICACIONS.versio%TYPE,
   p_dataPujada      IN  APLICACIONS.dataPujada%TYPE,
   p_URL IN APLICACIONS.URL%TYPE,
   p_resolucio IN APLICACIONS.resolucio%TYPE,
   p_activa IN APLICACIONS.activa%TYPE,
   RSP          OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_MODIFICACIO_APLICACIONS
   Propsit:            modifica dades d'una aplicaci a la taula APLICACIONS 
                   
   Parmetres entrada:  p_idAplicacio, p_versio, p_dataPujada, p_enllacBinari,
			 p_URL, p_resolucio, p_activa
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
paramEntrada  VARCHAR2(500);
existeix          INTEGER;
err_msg           VARCHAR2(200);
camp_nul          VARCHAR2(20);
E_NUL             EXCEPTION;
E_NO_EXISTEIX     EXCEPTION;
BEGIN
  paramEntrada:= 'p_idAplicacio: '||p_idAplicacio||
                 ', p_versio: '||p_versio||
                 ', p_dataPujada: '||p_dataPujada||
                 ', p_URL: '||p_URL||
                 ', p_resolucio: '||p_resolucio||
                 ', p_activa: '||p_activa;

  --verifica que els camps obligatoris no siguin nuls
   IF (p_idAplicacio IS NULL) THEN 
     camp_nul:= 'idAplicacio';
     RAISE E_NUL;
   ELSIF (p_versio IS NULL) THEN 
     camp_nul:= 'versio';
     RAISE E_NUL;
   ELSIF (p_dataPujada IS NULL) THEN 
     camp_nul:= 'dataPujada';
     RAISE E_NUL;
  ELSIF (p_resolucio IS NULL) THEN 
     camp_nul:= 'resolucio';
     RAISE E_NUL;
  ELSIF (p_activa IS NULL) THEN 
     camp_nul:= 'activa';
     RAISE E_NUL;

  END IF; 

  
   -- modifica les dades de l'aplicacio
   UPDATE APLICACIONS 
   SET versio= p_versio, 
       dataPujada = p_dataPujada,
       URL= p_URL,
       resolucio= p_resolucio,
       activa= p_activa

   WHERE idAplicacio = p_idAplicacio;
     
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_APLICACIONS', paramEntrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_APLICACIONS', paramEntrada, 'RSP: '||RSP);
            
   WHEN E_NO_EXISTEIX THEN
        RSP:= 'ERROR: aplicacio no existeix';
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_APLICACIONS', paramEntrada, 'RSP: '||RSP);
            
   
   WHEN OTHERS THEN
        ROLLBACK;
        
        IF (SQLCODE = -00001) THEN 
          RSP:= 'ERROR: ja existeix una aplicacio amb mateix: codi identificactiu';
          SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_APLICACIONS', paramEntrada, 'RSP: '||RSP);
          
        ELSE 
          err_msg := SUBSTR (SQLERRM, 1, 200);
          RSP:=    'ERROR: ' || err_msg;
          SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_APLICACIONS', paramEntrada, 'RSP: '||RSP);
          
        END IF;
              
END SP_MODIFICACIO_APLICACIONS;