create or replace prOCEDURE SP_ALTA_DESCARREGUES (
   p_aplicacio IN DESCARREGUES.aplicacio%TYPE,
   p_dispositiu IN DESCARREGUES.dispositiu%TYPE,
   p_dataDEsc IN DESCARREGUES.dataDesc%TYPE,
   p_modePagament IN DESCARREGUES.modePagament%TYPE,
   p_preu IN DESCARREGUES.preu%TYPE,
   p_operador IN DESCARREGUES.operador%TYPE,
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_ALTA_DESCARREGUES
   Propsit:            afegeix una descarrega a la taula DESCARREGUES
                   
   Parmetres entrada:  p_aplicacio , p_dispositiu , p_dataDesc , p_modePagament, p_preu,
    p_operador
                 
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
paramEntrada VARCHAR2(500);
existeix INTEGER;
existeix_fk1 INTEGER;
existeix_fk2 INTEGER;
err_msg VARCHAR2(200);
camp_nul VARCHAR2(20);
p_activa CHAR(2);
p_usuari NUMBER;
p_pais CHAR(2);
E_NUL EXCEPTION;
E_FK1 EXCEPTION;
E_FK2 EXCEPTION;
E_NO_ACTIVA EXCEPTION;
E_JA_EXISTEIX EXCEPTION;
BEGIN
   
   paramEntrada:='p_aplicacio: '||p_aplicacio||
                ', p_dispositiu: '||to_char(p_dispositiu)||
                ', p_dataDesc: '||to_char(p_dataDesc)||
                ', p_modePagament: '||p_modePagament||
                ', p_preu: '||to_char(p_preu)||
                ', p_operador: '||p_operador;
   
  -- verifica que els camps obligatoris no iguin nulos
   IF (p_aplicacio IS NULL) THEN 
     camp_nul:='aplicacio';
     RAISE E_NUL;
   ELSIF (p_dispositiu IS NULL) THEN 
     camp_nul:='dispositiu';
     RAISE E_NUL;
   ELSIF (p_dataDesc IS NULL) THEN 
     camp_nul:='dataDesc';
     RAISE E_NUL;
   ELSIF (p_modePagament IS NULL) THEN 
     camp_nul:='modePagament';
     RAISE E_NUL;
   ELSIF (p_preu IS NULL) THEN 
     camp_nul:='preu';
     RAISE E_NUL;
   ELSIF (p_operador IS NULL) THEN 
     camp_nul:='operador';
     RAISE E_NUL;
   END IF; 

  --verifica si existeix l' aplicacio
   SELECT COUNT (*)
   INTO existeix_fk1
   FROM APLICACIONS
   WHERE idAplicacio=p_aplicacio;

   IF (existeix_fk1=0) THEN
     RAISE E_FK1;
   END IF;

  --verifica si l'aplicacio est disponible per descarregar
   SELECT activa
   INTO p_activa
   FROM APLICACIONS
   WHERE idAplicacio=p_aplicacio;

   IF (p_activa='NO') THEN
     RAISE E_NO_ACTIVA;
   END IF;

  --verifica si existeix el dispositiu
   SELECT COUNT (*)
   INTO existeix_fk2
   FROM MODELSDISPOSITIU
   WHERE codiImei=p_dispositiu;

   IF (existeix_fk2=0) THEN
     RAISE E_FK2;
   END IF;

  --verifica si existeix la descarrega
   SELECT COUNT (*)
   INTO existeix
   FROM DESCARREGUES
   WHERE aplicacio=p_aplicacio AND dispositiu=p_dispositiu;

   IF (existeix > 0) THEN
     RAISE E_JA_EXISTEIX;
   END IF;

  --donem a p_usuari el valor de l'usuari del dispositiu
   SELECT usuari
   INTO p_usuari
   FROM MODELSDISPOSITIU
   WHERE codiimei=p_dispositiu;

  --donem a p_pais el valor del pais de l'usuari
   SELECT pais
   INTO p_pais
   FROM USUARI
   WHERE nmobil=p_usuari;
   
  -- insertem la nova entrada en la taula DESCARREGUES
   INSERT INTO DESCARREGUES (aplicacio, dispositiu, usuari, pais, dataDesc, modePagament, preu, operador)
   VALUES (p_aplicacio, p_dispositiu, p_usuari, p_pais, p_dataDesc, p_modePagament, p_preu, p_operador);

--------------------------Actualitzem EST1-------------------------
   SP_ALTA_MODIF_EST1();
--------------------------------------------------------------------

--------------------------Actualitzem EST2-------------------------
   SP_ALTA_MODIF_EST2();
--------------------------------------------------------------------  

--------------------------Actualitzem EST3-------------------------

   SP_ALTA_MODIF_EST3(EXTRACT(YEAR FROM p_dataDesc));
--------------------------------------------------------------------  

--------------------------Actualitzem EST4-------------------------

   SP_ALTA_MODIF_EST4(EXTRACT(YEAR FROM p_dataDesc));
-------------------------------------------------------------------- 

--------------------------Actualitzem EST5-------------------------

   SP_ALTA_MODIF_EST5(EXTRACT(YEAR FROM p_dataDesc));
-------------------------------------------------------------------- 

--------------------------Actualitzem EST6-------------------------

   SP_ALTA_MODIF_EST6(EXTRACT(YEAR FROM p_dataDesc), p_pais);
-------------------------------------------------------------------- 

--------------------------Actualitzem EST7-------------------------

   SP_ALTA_MODIF_EST7(EXTRACT(YEAR FROM p_dataDesc), p_pais);
-------------------------------------------------------------------- 

--------------------------Actualitzem EST8-------------------------

   SP_ALTA_MODIF_EST8(EXTRACT(YEAR FROM p_dataDesc), p_pais);
-------------------------------------------------------------------- 
   
  -- finaliza con xito
   RSP:='OK';
   SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESCARREGUES', paramEntrada, 'RSP: '||RSP);
   COMMIT;
   
EXCEPTION
  -- no finalitza amb xit perque algn valor obligatori es nul
   WHEN E_NUL THEN
        RSP:='ERROR: el camp '||camp_nul||' no por ser nulo'; 

        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESCARREGUES', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza amb xit perque no existeix la aplicacio indicada   
   WHEN E_FK1 THEN
        RSP:='ERROR: la aplicacio no existeix';
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESCARREGUES', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;ROLLBACK;
  -- no finalitza amb xito perque no existeix el dispositiu indicat   
   WHEN E_FK2 THEN
        RSP:='ERROR: el dispositiu no existeix';
       SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESCARREGUES', paramEntrada, 'RSP: '||RSP);
       ROLLBACK; ROLLBACK;
  -- no finaliza con xito porque la aplicacion indicada no est disponible para DESCARREGUES  
   WHEN E_NO_ACTIVA THEN
        RSP:='ERROR: la aplicacio no es troba disponible per descarregar';
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESCARREGUES', paramEntrada, 'RSP: '||RSP);   
        ROLLBACK;ROLLBACK; 
  -- no finalitza amb xit  perque la descarrega ja existeix
        RSP:='ERROR: la descarrega ja existeix';
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESCARREGUES', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;ROLLBACK; ROLLBACK;
  -- no finalitza amb xit por algn altre motiu 
   WHEN OTHERS THEN
        err_msg :=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESCARREGUES', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;ROLLBACK;
      
END SP_ALTA_DESCARREGUES;
