create or replace PROCEDURE SP_BAIXA_DESCARREGUES (
   p_aplicacio IN DESCARREGUES.aplicacio%TYPE,
   p_dispositiu IN DESCARREGUES.dispositiu%TYPE,
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_BAIXA_DESCARREGUES

   Propsit :           Elimina una descarrega del sistema. Actualitza les 
			taules del mdul estadstic.
                   
   Parmetres entrada:  p_aplicacio, p_dispositiu

   Parmetres sortida : RSP          

   Retorna:             'OK' si la execuci ha finalitzat amb xit
                        'ERROR: tipus de error' si ha fracassat
******************************************************************************/
paramEntrada VARCHAR2(500);
existeix INTEGER;
err_msg VARCHAR2(200);
camp_nul VARCHAR2(20);
p_dataDesc DATE;
p_pais CHAR(2);
E_NUL EXCEPTION;
E_NO_EXISTEIX EXCEPTION;
BEGIN
  
   paramEntrada:='p_aplicacio: '||p_aplicacio||
                ', p_dispositiu: '||to_char(p_dispositiu);
         
  -- verifica que el camp obligatori no sigui nul
   IF (p_aplicacio IS NULL) THEN 
     camp_nul:='aplicacio';
     RAISE E_NUL;
   ELSIF (p_dispositiu IS NULL) THEN 
     camp_nul:='dispositiu';
     RAISE E_NUL;
   END IF; 

  --verifica si existeix la descarrega
   SELECT COUNT (*)
   INTO existeix
   FROM DESCARREGUES
   WHERE aplicacio=p_aplicacio AND dispositiU=p_dispositiU;

   IF (existeIX=0) THEN
     RAISE E_NO_EXISTEIX;
   END IF;

  --donem p_dataDesc i p_pais el valor de la data i el pais de la descarrega
  --para utilizar-los per actualitzar el mdul estadstic
   SELECT dataDesc, pais
   INTO p_dataDesc, p_pais
   FROM DESCARREGUES
   WHERE aplicacio=p_aplicacio AND dispositiu=p_dispositiu;

  -- eliminem l'entrada de la taula
   DELETE FROM DESCARREGUES
   WHERE aplicacio=p_aplicacio AND dispositiu=p_dispositiu;

--------------------------Actualitzem EST1-------------------------
   SP_ALTA_MODIF_EST1();
--------------------------------------------------------------------

--------------------------Actualitzem EST2-------------------------
   SP_ALTA_MODIF_EST2();
--------------------------------------------------------------------  

--------------------------Actualitzem EST3-------------------------

   SP_ALTA_MODIF_EST3(EXTRACT(YEAR FROM p_dataDesc));
--------------------------------------------------------------------

--------------------------Actualitzem EST4-------------------------

   SP_ALTA_MODIF_EST4(EXTRACT(YEAR FROM p_dataDesc));
-------------------------------------------------------------------- 

--------------------------Actualitzem EST5-------------------------

   SP_ALTA_MODIF_EST5(EXTRACT(YEAR FROM p_dataDesc));
-------------------------------------------------------------------- 

--------------------------Actualitzem EST6-------------------------

   SP_ALTA_MODIF_EST6(EXTRACT(YEAR FROM p_dataDesc), p_pais);
-------------------------------------------------------------------- 

--------------------------Actualitzem EST7-------------------------

   SP_ALTA_MODIF_EST7(EXTRACT(YEAR FROM p_dataDesc), p_pais);
-------------------------------------------------------------------- 

--------------------------Actualitzem EST8-------------------------

   SP_ALTA_MODIF_EST8(EXTRACT(YEAR FROM p_dataDesc), p_pais);
-------------------------------------------------------------------- 

  -- finalitza amb xit  
   RSP:='OK';
   SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_DESCARREGUES', paramEntrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
  -- no finalitza amb xit perque algn valor obligatori es nul
   WHEN E_NUL THEN
        RSP:='ERROR: el camp ' || camp_nul || ' no pot ser nul';
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_DESCARREGUES', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finalitza amb xit perque no existeix l'entrada que es vol eliminar    
   WHEN E_NO_EXISTEIX THEN
        RSP:='ERROR: no existe la entrada que se quiere eliminar';
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_DESCARREGUES', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finalitza amb xit per algn altre motiu  
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: ' || err_msg;
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_DESCARREGUES', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;
        
END SP_BAIXA_DESCARREGUES;
