create or replace PROCEDURE SP_ALTA_PREU (
   p_aplicacio IN PREU.aplicacio%TYPE,
   p_pais IN PREU.pais%TYPE,
   p_preu IN PREU.preu%TYPE,
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:              SP_ALTA_PREU

   Propsit:           Afegeix una entrada a la taula preu
                   
   Parmetres entrada:  p_aplicacio, p_pais, p_preu

   Parmetres sortida: RSP          

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus de error' si ha fracassat
******************************************************************************/
paramEntrada VARCHAR2(500);
existeix INTEGER;
existeix_fk1 INTEGER;
existeix_fk2 INTEGER;
err_msg VARCHAR2(200);
camp_nul VARCHAR2(20);
E_NUL EXCEPTION;
E_JA_EXISTEIX EXCEPTION;
E_FK EXCEPTION;
E_FK1 EXCEPTION;
E_FK2 EXCEPTION;
BEGIN
   
   paramEntrada:='p_aplicacio: '||p_aplicacio||
                ', p_pais: '||p_pais||
                ', p_preu: '||to_char(p_preu);
   
   
  -- verifica que els camps obligatoris no siguin nuls
   IF (p_aplicacio IS NULL) THEN 
     camp_nul:='aplicacio';
     RAISE E_NUL;
   ELSIF (p_pais IS NULL) THEN 
     camp_nul:='pais';
     RAISE E_NUL;
   ELSIF (p_preu IS NULL) THEN 
     camp_nul:='preu';
     RAISE E_NUL; 
   END IF; 

  --verifica si existeixen l'aplicaci i el pas
   SELECT COUNT (*)
   INTO existeix_fk1
   FROM APLICACIONS
   WHERE idAplicacio=p_aplicacio;

   SELECT COUNT (*)
   INTO existeix_fk2
   FROM PAIS
   WHERE idPais=p_pais;

   IF (existeix_fk1=0 AND existeix_fk2=0) THEN
     RAISE E_FK;
   ELSIF (existeix_fk1=0) THEN 
     RAISE E_FK1;
   ELSIF (existeix_fk2=0) THEN 
     RAISE E_FK2;
   END IF;

  --verifica si existeix el preu
   SELECT COUNT (*)
   INTO existeix
   FROM PREU
   WHERE aplicacio=p_aplicacio AND pais=p_pais;

   IF (existeix > 0) THEN
     RAISE E_JA_EXISTEIX;
   END IF;

  -- insertem la nova entrada a la taula preu
   INSERT INTO PREU (aplicacio, pais, preu)
   VALUES (p_aplicacio, p_pais, p_preu);
   
  -- finaltiza amb xit
   RSP:='OK';
   SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_PREU', paramEntrada, 'RSP: '||RSP);

   COMMIT;
EXCEPTION
  -- no finalitza amb xit perque algn valor obligatori s nul
   WHEN E_NUL THEN
        RSP:='ERROR: el camp '||camp_nul||' no potser nul'; 
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_PREU', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finalitza amb xit perque no existeixen l' aplicaci ni el pas      
   WHEN E_FK THEN
        RSP:='ERROR: la aplicacio i el pais no existeixen';
       SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_PREU', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;       
  -- no finaltiza amb xito perque no existeix l'aplicaci     
   WHEN E_FK1 THEN
        RSP:='ERROR: la aplicacio no existeix';
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_PREU', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito porque no existe el pas indicado     
   WHEN E_FK2 THEN
        RSP:='ERROR: el pais no existe';
         SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_PREU', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;  
  -- no finalitza amb xito perque ja existeix el preu    
   WHEN E_JA_EXISTEIX THEN
        RSP:='ERROR: el preciou ja existeix';
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_PREU', paramEntrada, 'RSP: '||RSP);
        ROLLBACK; 
  -- no finalitza amb xit por algn altre motiu     
   WHEN OTHERS THEN         
        err_msg :=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_PREU', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;         
              
END SP_ALTA_PREU;