create or replace PROCEDURE SP_ALTA_DESCRIPCIO (
   p_idioma IN DESCRIPCIO.idioma%TYPE,
   p_aplicacio IN DESCRIPCIO.aplicacio%TYPE,
   p_descripcio IN DESCRIPCIO.descripcio%TYPE,
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:              PR_ALTA_DESCRIPCIO

   Propsit:           Afegeix una entrada a la taula descripcio
                   
   Parmetres entrada:  p_idioma, p_aplicacio, p_descripcio

   Parmetros sortida: RSP          

   Retorna:             'OK' si l'execucio ha finalizat amb xit
                        'ERROR: tipus de error' si ha fracassat
******************************************************************************/
paramEntrada VARCHAR2(500);
existeix INTEGER;
existeix_fk INTEGER;
err_msg VARCHAR2(200);
camp_nul VARCHAR2(20);
E_NUL EXCEPTION;
E_JA_EXISTEIX EXCEPTION;
E_FK EXCEPTION;
BEGIN
  
   paramEntrada:='p_idioma: '||p_idioma||
                ', p_aplicacio: '||p_aplicacio||
                ', p_descripcio: '||p_descripcio;
                 

  -- verifica que els camps obligatoris no siguin nuls
   IF (p_idioma IS NULL) THEN 
     camp_nul:='idioma';
     RAISE E_NUL;
   ELSIF (p_aplicacio IS NULL) THEN 
     camp_nul:='aplicacio';
     RAISE E_NUL;
   ELSIF (p_descripcio IS NULL) THEN 
     camp_nul:='descripcio';
     RAISE E_NUL; 
   END IF; 

  --verifica si existeix l'aplicaci
   SELECT COUNT (*)
   INTO existeix_fk
   FROM APLICACIONS
   WHERE idAplicacio=p_aplicacio;

   IF (existeix_fk=0) THEN
     RAISE E_FK;
   END IF;

  --verifica si existeix la descripci
   SELECT COUNT (*)
   INTO existeix
   FROM DESCRIPCIO
   WHERE idioma=p_idioma AND aplicacio=p_aplicacio;

   IF (existeix > 0) THEN
     RAISE E_JA_EXISTEIX;
   END IF;

  -- insertem la nova entrada a la taula descripci
   INSERT INTO DESCRIPCIO (idioma, aplicacio, descripcio)
   VALUES (p_idioma, p_aplicacio, p_descripcio);
   
  -- finalitza amb xit
   RSP:='OK';
   SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESCRIPCIO', paramEntrada, 'RSP: '||RSP);

   COMMIT;
EXCEPTION
  -- no finalitza amb xit perque algn valor obligatori s nul
   WHEN E_NUL THEN
        RSP:='ERROR: el camp '||camp_nul||' no pot ser nul';  
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESCRIPCIO', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finalitza amb xit perque no existeix l'aplicaci   
   WHEN E_FK THEN
        RSP:='ERROR: no existeix la aplicaci';
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESCRIPCIO', paramEntrada, 'RSP: '||RSP);
        ROLLBACK; 
  -- no finalitza amb xit perque ja existeix una descripci per la mateixa aplicaci i idioma 
   WHEN E_JA_EXISTEIX THEN
        RSP:='ERROR: la descripcio ja existeix';
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESCRIPCIO', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;   
  -- no finalitza amb xit per algn altre motiu    
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESCRIPCIO', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;       
              
END SP_ALTA_DESCRIPCIO;