create or replace PROCEDURE SP_ALTA_DISPONIBILITAT (
   p_aplicacio IN DISPONIBILITAT.aplicacio%TYPE,
   p_sistema_operatiu IN DISPONIBILITAT.soperatiu%TYPE,
   p_mida IN DISPONIBILITAT.mida%TYPE,
   p_enllacbinari IN DISPONIBILITAT.enllacbinari%TYPE,
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:              SP_ALTA_DISPONIBILITAT

   Propsit:           Afegeix una entrada a la taula DISPONIBILITAT
                   
   Parmetres entrada:  p_aplicacio, p_sistema_operatiu, p_mida, p_enllacbinari

   Parmetres sortida : RSP          

   Retorna:             'OK' si l'execucio ha finalitzat amb xito
                        'ERROR: tipus de error' si ha fracassat
******************************************************************************/
paramEntrada VARCHAR2(500);
existeix INTEGER;
existeix_fk1 INTEGER;
existeix_fk2 INTEGER;
err_msg VARCHAR2(200);
camp_nul VARCHAR2(20);
E_NUL EXCEPTION;
E_JA_EXISTEIX EXCEPTION;
E_FK EXCEPTION;
E_FK1 EXCEPTION;
E_FK2 EXCEPTION;
BEGIN
  
   paramEntrada:='p_aplicacio: '||p_aplicacio||
                ', p_sistema_operatiu: '||p_sistema_operatiu||
                ', p_mida: '||to_char(p_mida)||
                ', p_enllacbinari: '||p_enllacbinari;
   
   
  -- verifica que els camps obligatoris no siguin nulos
   IF (p_aplicacio IS NULL) THEN 
     camp_nul:='aplicacio';
     RAISE E_NUL;
   ELSIF (p_sistema_operatiu IS NULL) THEN 
     camp_nul:='sistema_operatiu';
     RAISE E_NUL;
   ELSIF (p_mida IS NULL) THEN 
     camp_nul:='mida';
     RAISE E_NUL; 
   ELSIF (p_enllacbinari IS NULL) THEN 
     camp_nul:='enllacbinari';
     RAISE E_NUL;
   END IF; 

  --verifica si existeixen l'aplicaci i el sistema operatiu
   SELECT COUNT (*)
   INTO existeix_fk1
   FROM APLICACIONS
   WHERE idAplicacio=p_aplicacio;

   SELECT COUNT (*)
   INTO existeix_fk2
   FROM SOPERATIU
   WHERE idSO=p_sistema_operatiu;

   IF (existeix_fk1=0 AND existeix_fk2=0) THEN
     RAISE E_FK;
   ELSIF (existeix_fk1=0) THEN 
     RAISE E_FK1;
   ELSIF (existeix_fk2=0) THEN 
     RAISE E_FK2;
   END IF;

  --verifica si existeix l'entrada en la taula
   SELECT COUNT (*)
   INTO existeix
   FROM DISPONIBILITAT
   WHERE aplicacio=p_aplicacio AND soperatiu=p_sistema_operatiu;

   IF (existeix>0) THEN
     RAISE E_JA_EXISTEIX;
   END IF;
   
  -- insertem la nova entrada en la taula 
   INSERT INTO DISPONIBILITAT (aplicacio, soperatiu, mida, enllacbinari)
   VALUES (p_aplicacio, p_sistema_operatiu, p_mida, p_enllacbinari);
   
  -- finalitza amb xit
   RSP:='OK';
   SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DISPONIBILITAT', paramEntrada, 'RSP: '||RSP);
   COMMIT;
   
EXCEPTION
  -- no finalitza amb xit perque algn valor obligatori es nul
   WHEN E_NUL THEN
        RSP:='ERROR: el camp '||camp_nul||' no potser nul'; 
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DISPONIBILITAT', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finalitza amb xit perque no existeixen l'aplicaci ni el sistema operatiu    
   WHEN E_FK THEN
        RSP:='ERROR: la aplicacio i el sistema operatiu no existeixen';
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DISPONIBILITAT', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;       
  -- no finalitza amb xit perque no existeix l'aplicaci indicada     
   WHEN E_FK1 THEN
        RSP:='ERROR: la aplicaci no existeix';
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DISPONIBILITAT', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finalitza amb xit perque no existeix el sistema operatiu     
   WHEN E_FK2 THEN
        RSP:='ERROR: el sistema operatiu no existeix';
       SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DISPONIBILITAT', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza amb xit perque ja existeix aquesta entrada a la taula       
   WHEN E_JA_EXISTEIX THEN
        RSP:='ERROR: ja existeix aquesta entrada a la taula';
       SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DISPONIBILITAT', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finalitza amb xit per algun altre motiu 
   WHEN OTHERS THEN         
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DISPONIBILITAT', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;
      
END SP_ALTA_DISPONIBILITAT;