create or replace PROCEDURE SP_ALTA_DESENVOLUPA (
   p_desenvolupador IN DESENVOLUPA.desenvolupador%TYPE,
   p_aplicacio IN DESENVOLUPA.aplicacio%TYPE,
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_ALTA_DESENVOLUPA

   Propsit:           Afegeix una entrada ala taula DESENVOLUPA
                   
   Parmetres entrada:  p_desenvolupador, p_aplicacio

   Parmetres sortida: RSP          

   Retorna:             'OK' si l'execucio hafinalitzat amb xit
                        'ERROR: tipus de error' si ha fracassat
******************************************************************************/
paramEntrada VARCHAR2(500);
existeix INTEGER;
existeix_fk1 INTEGER;
existeix_fk2 INTEGER;
err_msg VARCHAR2(200);
camp_nul VARCHAR2(20);
E_NUL EXCEPTION;
E_FK EXCEPTION;
E_FK1 EXCEPTION;
E_FK2 EXCEPTION;
E_JA_EXISTEIX EXCEPTION;
BEGIN
  
   paramEntrada:='p_desenvolupador: '||to_char(p_desenvolupador)||
                ', p_aplicacio: '||p_aplicacio;   
   
  -- verifica que els campo obligatoris no siguin nuls
   IF (p_desenvolupador IS NULL) THEN 
     camp_nul:='desenvolupador';
     RAISE E_NUL;
   ELSIF (p_aplicacio IS NULL) THEN 
     camp_nul:='aplicacio';
     RAISE E_NUL;
   END IF; 

  --verifica si existeixen el desenvolupador i la aplicaci
   SELECT COUNT (*)
   INTO existeix_fk1
   FROM DESENVOLUPADOR
   WHERE idDesenvolupador=p_desenvolupador;

   SELECT COUNT (*)
   INTO existeix_fk2
   FROM APLICACIONS
   WHERE idAplicacio=p_aplicacio;

   IF (existeix_fk1=0 AND existeix_fk2=0) THEN
     RAISE E_FK;
   ELSIF (existeix_fk1=0) THEN 
     RAISE E_FK1;
   ELSIF (existeix_fk2=0) THEN 
     RAISE E_FK2;
   END IF;

  --verifica si existeix la entrada a la taula
  SELECT COUNT (*)
   INTO existeix
   FROM DESENVOLUPA
   WHERE desenvolupador=p_desenvolupador AND aplicacio=p_aplicacio;

   IF (existeix > 0) THEN
     RAISE E_JA_EXISTEIX;
   END IF;
   
  -- insertem la nova entrada a la taula
   INSERT INTO DESENVOLUPA (desenvolupador, aplicacio)
   VALUES (p_desenvolupador, p_aplicacio);
   
  -- finalitza amb xit
   RSP:='OK';
   SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESENVOLUPA', paramEntrada, 'RSP: '||RSP);
   COMMIT;
   
EXCEPTION
  -- no finalitza amb xit perque algn valor obligatori es nul
   WHEN E_NUL THEN
        RSP:='ERROR: el camp '||camp_nul||' no pot ser nul'; 
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESENVOLUPA', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finalitza amb xito perque no existeixen el desenvolupador ni l'aplicaci     
   WHEN E_FK THEN
        RSP:='ERROR: el desenvolupador i la aplicacion no existeixen';
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESENVOLUPA', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;        
  -- no finalitza amb xit perque no existeix el desenvolupador   
   WHEN E_FK1 THEN
        RSP:='ERROR: el desenvolupador no existeix';
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESENVOLUPA', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finalitza amb xit perque no existeix la aplicacio     
   WHEN E_FK2 THEN
        RSP:='ERROR: la aplicacio no existeix';
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESENVOLUPA', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;  
  -- no finalitza amb xit perque l'entrada ja existeix a la taula
   WHEN E_JA_EXISTEIX THEN
        RSP:='ERROR: la entrada ja existeix';
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESENVOLUPA', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finalitza amb xit per algun altre motiu     
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DESENVOLUPA', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;
      
END SP_ALTA_DESENVOLUPA;