create or replace PROCEDURE SP_ALTA_MODELSDISPOSITIU (
   p_codiImei IN MODELSDISPOSITIU.codiImei%TYPE,
   p_sistema_operatiU IN MODELSDISPOSITIU.soperatiu%TYPE,
   p_modelDispositiu IN MODELSDISPOSITIU.modelDispositiu%TYPE,
   p_resolucio IN MODELSDISPOSITIU.resolucio%TYPE,
   p_usuari IN MODELSDISPOSITIU.usuari%TYPE,
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nome:              SP_ALTA_MODELSDISPOSITIU

   Propsit:           Afegeix un nou dispositiu
                   
   Parmetres entrada:  p_codiImei, p_sistema_operatiu, p_modelDispositiu, p_resolucio, 
			p_usuari

   Parmetres sortida: RSP          

   Retorna:             'OK' si l'execuci ha finalizat amb xit
                        'ERROR: tipus de error' si ha fracassat
******************************************************************************/
paramEntrada VARCHAR2(500);
existeix INTEGER;
existeix_fk INTEGER;
err_msg VARCHAR2(200);
camp_nul VARCHAR2(20);
E_NUL EXCEPTION;
E_FK EXCEPTION;
E_JA_EXISTEIX EXCEPTION;
BEGIN
  
   paramEntrada:='p_codiImei: '||to_char(p_codiImei)||
                  ', p_sistema_operatiu: '||p_sistema_operatiu||
                  ', p_modelDispositiu: '||p_modelDispositiu||
                  ', p_resolucio: '||p_resolucio||
                  ', p_usuari: '||to_char(p_usuari);
   
  -- verifiquem que els camps obligatoris no siguin nuls
   IF (p_codiImei IS NULL) THEN 
     camp_nul:='codiImei';
     RAISE E_NUL;
   ELSIF (p_sistema_operatiu IS NULL) THEN 
     camp_nul:='sistema_operatiu';
     RAISE E_NUL;
   ELSIF (p_modelDispositiu IS NULL) THEN 
     camp_nul:='modelDispositiu';
     RAISE E_NUL; 
   ELSIF (p_resolucio IS NULL) THEN 
     camp_nul:='resolucio';
     RAISE E_NUL;
   ELSIF (p_usuari IS NULL) THEN 
     camp_nul:='usuari';
     RAISE E_NUL;
   END IF; 

  --verifica si existeix l'usuari
   SELECT COUNT (*)
   INTO existeix_fk
   FROM USUARI
   WHERE nmobil=p_usuari;

   IF (existeix_fk=0) THEN
     RAISE E_FK;
   END IF;

  --verifica si existeix el model dispositiu
   SELECT COUNT (*)
   INTO existeix
   FROM MODELSDISPOSITIU
   WHERE codiImei=p_codiImei;

   IF (existeix > 0) THEN
     RAISE E_JA_EXISTEIX;
   END IF;
   
  -- inserta la nova entrada a la taula MODELSDISPOSITIU
   INSERT INTO MODELSDISPOSITIU(codiImei, soperatiu, modelDispositiu, resolucio, usuari)
   VALUES (p_codiImei, p_sistema_operatiu, p_modelDispositiu, p_resolucio, p_usuari);
   
  -- finalitza amb xit
   RSP:='OK';
   SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODELSDISPOSITIU', paramEntrada, 'RSP: '||RSP);
   COMMIT;
   
EXCEPTION
  -- no finalitza amb xit perque algn valor obligatori es nul
   WHEN E_NUL THEN
        RSP:='ERROR: el camp '||camp_nul||' no pot ser nul'; 
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODELSDISPOSITIU', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finalitza amb xit perque no existeix l'usuari 
   WHEN E_FK THEN
        RSP:='ERROR: el usuari no existeix';
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODELSDISPOSITIU', paramEntrada, 'RSP: '||RSP);  
        ROLLBACK;
  -- no finalitza amb xit perque el dispositiu ja existeix 
   WHEN E_JA_EXISTEIX THEN
        RSP:='ERROR: el dispositiu ya existeix';
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODELSDISPOSITIU', paramEntrada, 'RSP: '||RSP);
        ROLLBACK; 
  -- no finalitza amb xit per algn altre motiu    
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
         SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODELSDISPOSITIU', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;
      
END SP_ALTA_MODELSDISPOSITIU;