create or replace PROCEDURE FU_LLISTAT_DESENVOLUPADOR (
   p_pais IN PAIS.idPais%TYPE,
   REFCURSOR OUT SYS_REFCURSOR, 
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
      Nom:                 FU_LLISTAT_DESENVOLUPADOR
   
      Propsit:            El llistat dels desenvolupadors d'un pas donat amb 
                           totes les seves aplicacions publicades
      
                   
      Parmetres entrada:  p_pais  

      Parmetres de sortida: REFCURSOR, RSP          

      Retorna:             'OK' si la ejecucin ha finalizado con xito
                           'ERROR: tipo de error' si ha fracasado
******************************************************************************/
paramEntrada VARCHAR2(500);
num          INTEGER;
err_msg      VARCHAR2(200);
camp_nul     VARCHAR2(20);
E_NUL       EXCEPTION;
    
BEGIN
  
   paramEntrada:= ', p_pais: '||p_pais;

  -- verifiquem que el camp obligatorio no sigui nul
   IF (p_pais IS NULL) THEN 
     camp_nul:= 'pais';
     RAISE E_NUL;
   END IF;


  OPEN REFCURSOR FOR
  SELECT d.idDesenvolupador, d.nom, d.adreca, d.telefon, COUNT(des.aplicacio) AS num
  FROM DESENVOLUPADOR d 
  JOIN DESENVOLUPA des ON d.idDesenvolupador=des.desenvolupador
  WHERE d.pais=p_pais
  GROUP BY d.idDesenvolupador, d.nom, d.adreca, d.telefon, num;
  
  -- finaltiza amb xito
   RSP:= 'OK';
   SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_LLISTAT_DESENVOLUPADOR', paramEntrada, 'RSP: '||RSP);
  
  COMMIT;
  
EXCEPTION
  -- no finaltiza amb xito perque el valor obligatori es nul
   WHEN E_NUL THEN
        RSP:='ERROR: el camp pais no pot ser nul'; 
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_LLISTAT_DESENVOLUPADOR', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;

  -- no finalitza amb xito per lagun altre motiu         
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_LLISTAT_DESENVOLUPADOR', paramEntrada, 'RSP: '||RSP);
        ROLLBACK; 
   
END FU_LLISTAT_DESENVOLUPADOR;