create or replace PROCEDURE FU_LLISTAT_PAISOS_APLICACIO (
   p_aplicacio IN APLICACIONS.idAplicacio%TYPE,
   p_any NUMBER,
   REFCURSOR OUT SYS_REFCURSOR, 
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                FU_LLISTAT_PAISOS_APLICACIO

   Propsit:           Donada una aplicaci i un any concret: el llistat de tots 
		       pasos on s'ha descarregat aquell any any, aix com el 
		       nmero de descrregues que ha tingut cada pas
                   
   Parmetres entrada:  p_aplicacio, p_any  

   Parmetres de sortida: REFCURSOR, RSP          

   Retorna:             'OK' si l'execuci ha estat unxit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
paramEntrada   VARCHAR2(500);
num            INTEGER;
err_msg        VARCHAR2(200);
camp_nul       VARCHAR2(20);
E_NUL           EXCEPTION;
    
BEGIN
  
   paramEntrada:= 'p_aplicacio: '||p_aplicacio||
		  ', p_any: '||to_char(p_any);

  -- verifiquem que els camps obligatoris no siguin nuls
   IF (p_aplicacio IS NULL) THEN 
     camp_nul:='aplicacio';
     RAISE E_NUL;
   ELSIF (p_any IS NULL) THEN 
     camp_nul:='any';
     RAISE E_NUL;
   END IF;


  OPEN REFCURSOR FOR
  SELECT pais, COUNT(dataDesc) AS num
  FROM DESCARREGUES d
  WHERE aplicacio=p_aplicacio AND EXTRACT(YEAR FROM d.dataDesc)=p_any
  GROUP BY pais, num
  ORDER BY num DESC;
  
  -- finaltiza amb xit
   RSP:= 'OK';
   SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_LLISTAT_PAISOS_APLICACIO ', paramEntrada, 'RSP: '||RSP);
  
  COMMIT;
  
EXCEPTION
  -- no finaltiza amb xit perque algun valor oblifgatori es nul
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp '||camp_nul||' no pot ser nul'; 
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_LLISTAT_PAISOS_APLICACIO ', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaltiza amb xit per algun altre motiu         
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_LLISTAT_PAISOS_APLICACIO ', paramEntrada, 'RSP: '||RSP);
        ROLLBACK; 
   
END FU_LLISTAT_PAISOS_APLICACIO ;