create or replace PROCEDURE FU_LLISTAT_USUARIS_ACTIVITAT (
   p_usuari IN USUARI.nmobil%TYPE,
   REFCURSOR OUT SYS_REFCURSOR, 
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                  FU_LLISTAT_USUARIS_ACTIVITAT

   Propsit:             Donat un usuari final, el llistat de tota la seva activitat
			 de descrregues a la plataforma, incloent data, apliacio
			 descarregada, preu que va pagar
                   
   Parmetres entrada:  p_usuari  

   Parmetres sortida:  REFCURSOR, RSP          

   Retorna:             'OK' si la ejecuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
paramEntrada         VARCHAR2(500);
err_msg              VARCHAR2(200);
camp_nul             VARCHAR2(20);
E_NUL                EXCEPTION;
    
BEGIN
 
   paramEntrada:=', p_usuari: '||to_char(p_usuari);

  -- verifiquem que el camp obligatori no sigui nul
   IF (p_usuari IS NULL) THEN 
     camp_nul:='usuari';
     RAISE E_NUL;
   END IF;


  OPEN REFCURSOR FOR
  SELECT * 
  FROM DESCARREGUES
  WHERE usuari=p_usuari
  ORDER BY dataDesc DESC;
  
  -- finaltza amb xit
   RSP:='OK';
   SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_LLISTAT_USUARIS_ACTIVITAT', paramEntrada, 'RSP: '||RSP);
  
  COMMIT;
  
EXCEPTION
  -- no finalitza amb xit perque el valor obligatori es nul
   WHEN E_NUL THEN
        RSP:='ERROR: el camp usuari no pot ser nul'; 
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_LLISTAT_USUARIS_ACTIVITAT', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;

  -- no finalitza amb xit per algun altre motiu         
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_LLISTAT_USUARIS_ACTIVITAT', paramEntrada, 'RSP: '||RSP);

        ROLLBACK; 
   
END FU_LLISTAT_USUARIS_ACTIVITAT;