create or replace PROCEDURE FU_TOP20_USUARIS (
   p_any NUMBER,
   REFCURSOR OUT SYS_REFCURSOR, 
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                  FU_TOP20_USUARUS
   Propsit:             Donat un any concret el llistat dels 20 usuaris que 
			 ms diners s'han gastat en aplicacions mbils, ordenat
			 de ms a menys.
                   
   Parmetres  entrada:  p_any  

   Parmetres  sortida: REFCURSOR, RSP          

   Retorna:             'OK' si l'execucio ha finalitzat amb xit
                        'ERROR: tipus de error' si ha fracassat
******************************************************************************/
paramEntrada    VARCHAR2(500);
suma            INTEGER;
err_msg         VARCHAR2(200);
camp_nul        VARCHAR2(20);
E_NUL           EXCEPTION;
    
BEGIN
 
   paramEntrada:=', p_any: '||to_char(p_any);

  -- verifiquem que el camp obligatori no sigui nul
   IF (p_any IS NULL) THEN 
     camp_nul:='any';
     RAISE E_NUL;
   END IF;


  OPEN REFCURSOR FOR
  SELECT usuari, suma
  FROM (SELECT d.usuari AS usuari, SUM(d.preu) AS suma FROM DESCARREGUES d JOIN USUARI u ON d.usuari=u.nmobil 
       WHERE EXTRACT(YEAR FROM d.dataDesc)=p_any GROUP BY usuari, suma ORDER BY suma DESC)
       
  WHERE rownum<=20;
  
  -- finalitza amb xit
   RSP:='OK';
   SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_TOP20_USUARIS', paramEntrada, 'RSP: '||RSP);
  
  COMMIT;
  
EXCEPTION
  -- no finaltiza amb xit perque el valor obligatori es nul
   WHEN E_NUL THEN
        RSP:='ERROR: el camp any no pot ser nul'; 
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_TOP20_USUARIS', paramEntrada, 'RSP: '||RSP);
        ROLLBACK;

  -- no finalitza amb xit per algun altre motiu         
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        SP_ALTA_LOG1(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_TOP20_USUARIS', paramEntrada, 'RSP: '||RSP);
        ROLLBACK; 
   
END FU_TOP20_USUARIS;